SET ECHO ON
-- Script Oracle SQL*plus de cration du schma SyLeRat 
-- Les TRIGGER ne sont pas tous implments

-- Cration des tables
CREATE TABLE Utilisateur
(idUtilisateur 			VARCHAR(10)	NOT NULL,
 motPasse	 			VARCHAR(10)	NOT NULL,
 nom 					VARCHAR(10)	NOT NULL,
 prnom 				VARCHAR(10)	NOT NULL,
 catgorieUtilisateur 	VARCHAR(14)  	NOT NULL
 	CHECK(catgorieUtilisateur IN ('employ', 'membre')),
 PRIMARY KEY (idUtilisateur)
)
/
CREATE TABLE Employ
(idUtilisateur 			VARCHAR(10)	NOT NULL,
 codeMatricule	 		CHAR(6)	NOT NULL,
 catgorieEmploy			VARCHAR(15) NOT NULL 
	CHECK(catgorieEmploy IN ('biblothcaire', 'commis')),
 PRIMARY KEY (idUtilisateur),
 UNIQUE (codeMatricule),
 FOREIGN KEY (idUtilisateur) REFERENCES Utilisateur
)
/
CREATE OR REPLACE TRIGGER AIEmployModifierCatgorie
AFTER INSERT ON Employ
REFERENCING
	NEW AS ligneAprs
FOR EACH ROW
BEGIN
	UPDATE 	Utilisateur
	SET 		catgorieUtilisateur = 'employ'
	WHERE 	idUtilisateur = :ligneAprs.idUtilisateur;
END AIEmployModifierCatgorie ;
/
CREATE OR REPLACE TRIGGER BDUEmployInterdiction
BEFORE DELETE OR UPDATE ON Employ
BEGIN
	raise_application_error(-20200,'DELETE et UPDATE interdit pour Employ');
END BDUEmployInterdiction;
/
CREATE TABLE Membre
(idUtilisateur 			VARCHAR(10)	NOT NULL,
 tlphoneRsidence	 	VARCHAR(15)	NOT NULL,
 PRIMARY KEY (idUtilisateur),
 FOREIGN KEY (idUtilisateur) REFERENCES Utilisateur
)
/
CREATE OR REPLACE TRIGGER AIMembreModifierCatgorie
AFTER INSERT ON Membre
REFERENCING
	NEW AS ligneAprs
FOR EACH ROW
BEGIN
	UPDATE 	Utilisateur
	SET 		catgorieUtilisateur = 'membre'
	WHERE 	idUtilisateur = :ligneAprs.idUtilisateur;
END AIMembreModifierCatgorie ;
/
CREATE OR REPLACE TRIGGER BDUMembreInterdiction
BEFORE DELETE OR UPDATE ON Membre
BEGIN
	raise_application_error(-20201,'DELETE et UPDATE interdit pour Membre');
END BDUMembreInterdiction;
/
CREATE TABLE MembreGnral
(noSequence 			NUMBER(10)		NOT NULL,
 nbMaxPrts	 			NUMBER(10)		DEFAULT 5 NOT NULL,
 dureMaxPrts			NUMBER(10)		DEFAULT 7 NOT NULL,
 PRIMARY KEY (noSequence)
)
/
CREATE OR REPLACE TRIGGER BDMembreGnral
BEFORE DELETE ON MembreGnral
BEGIN
	raise_application_error(-20201,'DELETE interdit pour MembreGnral');
END BDMembreGnral;
/
CREATE OR REPLACE TRIGGER BUMembreGnral
BEFORE UPDATE OF nbMaxPrts ON MembreGnral
FOR EACH ROW
WHEN (NEW.nbMaxPrts < OLD.nbMaxPrts)
BEGIN
	raise_application_error(-20212,'Interdit de diminuer nbMaxPrts');
END BUMembreGnral;
/
CREATE TABLE Editeur
(nomEditeur 			VARCHAR(20)	NOT NULL,
 ville 				VARCHAR(20)	NOT NULL,
 PRIMARY KEY (nomEditeur)
)
/
CREATE TABLE Catgorie
(code					VARCHAR(10)	NOT NULL,
 descripteur 			VARCHAR(20)	NOT NULL,
 codeParent				VARCHAR(10),
 PRIMARY KEY (code),
 FOREIGN KEY (codeParent) REFERENCES Catgorie
)
/
CREATE TABLE Livre
(ISBN 				CHAR(13)		NOT NULL,
 titre 				VARCHAR(50)		NOT NULL,
 anneParution 			NUMBER(4)		NOT NULL
CHECK(anneParution > 0),
 nomEditeur 			VARCHAR(20)	NOT NULL,
 code					VARCHAR(10)	NOT NULL,
 PRIMARY KEY (ISBN),
 FOREIGN KEY (nomEditeur) REFERENCES Editeur
 	DEFERRABLE INITIALLY DEFERRED,
 FOREIGN KEY (code) REFERENCES Catgorie
)
/
CREATE OR REPLACE TRIGGER BDULivreInterdiction
BEFORE DELETE OR UPDATE ON Livre
BEGIN
	raise_application_error(-20202,'DELETE et UPDATE interdit pour Livre');
END BDULivreInterdiction;
/
CREATE TABLE Exemplaire
(idExemplaire 			VARCHAR(10)		NOT NULL,
 dateAchat 				DATE			NOT NULL,
 statut 				VARCHAR(15)		NOT NULL
	CHECK(statut IN ('prt', 'disponible','retir')),
 ISBN 				CHAR(13)		NOT NULL,
 PRIMARY KEY (idExemplaire),
 FOREIGN KEY (ISBN) REFERENCES Livre
)
/
CREATE OR REPLACE TRIGGER BIEditeurAuMoinsUnLivre
BEFORE INSERT ON Editeur
FOR EACH ROW
DECLARE
	nbEditeur 		INTEGER;
	editeurSansLivre 	EXCEPTION;
BEGIN
	SELECT COUNT(*) INTO nbEditeur FROM Livre
	WHERE nomEditeur = :NEW.nomEditeur;
	IF nbEditeur = 0 THEN
		RAISE editeurSansLivre;
	END IF;

EXCEPTION
	WHEN editeurSansLivre THEN
		raise_application_error(-20110,'l''diteur doit avoir au moins un livre');
	WHEN	OTHERS THEN
		raise_application_error(-20105,'erreur interne');
END BIEditeurAuMoinsUnLivre;
/
CREATE TABLE PrtEnCours
(idExemplaire 			VARCHAR(10)	NOT NULL,
 datePrt 				DATE	DEFAULT SYSDATE NOT NULL,
 idUtilisateur 			VARCHAR(10)	NOT NULL,
 PRIMARY KEY (idExemplaire),
 FOREIGN KEY (idUtilisateur) REFERENCES Utilisateur,
 FOREIGN KEY (idExemplaire) REFERENCES Exemplaire
)
/
CREATE OR REPLACE TRIGGER BIPrtEnCoursVrifier
 BEFORE INSERT ON PrtEnCours
 FOR EACH ROW
DECLARE
	statutExemplaire 		Exemplaire.statut%TYPE;
	idUtilisateurMembre	Membre.idUtilisateur%TYPE;
	datePrt			PrtEnCours.datePrt%TYPE;
	nombrePrtsEnCours 	INTEGER;
	nombreMaximalPrts 	INTEGER;
	dureMaximalePrts 	INTEGER;
	exemplaireNonDisponible EXCEPTION;
	nombreMaximalAtteint 	EXCEPTION;
	prtEnRetard 		EXCEPTION;

	-- Curseur pour vrifier si membre
	CURSOR leMembre(unIdUtilisateur PrtEnCours.idUtilisateur%TYPE)IS
		SELECT idUtilisateur
		FROM Membre WHERE idUtilisateur = unIdUtilisateur;
	-- Curseur (CURSOR) PL/SQL pour itrer sur les prts

	CURSOR lignesPrts(unIdUtilisateur PrtEnCours.idUtilisateur%TYPE)IS
    		SELECT 	datePrt
    		FROM		PrtEnCours
    		WHERE 	idUtilisateur = unIdUtilisateur;
BEGIN

	--  Vrifier si statut de l'exemplaire = disponible
	SELECT 	statut
	INTO 		statutExemplaire
	FROM 		Exemplaire
	WHERE 	idExemplaire = :NEW.idExemplaire
	FOR UPDATE; /* pour srialisabilit*/

	IF statutExemplaire <> 'disponible' THEN
		RAISE exemplaireNonDisponible;
	END IF;

	-- Vrifier si l'emprunteur est un membre
	OPEN leMembre(:NEW.idUtilisateur);
	FETCH leMembre INTO idUtilisateurMembre;
	IF leMembre%FOUND THEN
	-- Vrifier si le nombre maximal d'emprunts est atteint ou si un retard existe
		SELECT	nbMaxPrts, dureMaxPrts
		INTO		nombreMaximalPrts, dureMaximalePrts
		FROM		MembreGnral;

		LOCK TABLE PrtEnCours IN SHARE ROW EXCLUSIVE MODE;
		nombrePrtsEnCours := 0;

		OPEN lignesPrts(:NEW.idUtilisateur);
		LOOP
			FETCH lignesPrts INTO datePrt;
			EXIT WHEN lignesPrts%NOTFOUND;
			IF (SYSDATE-datePrt > dureMaximalePrts) THEN
				RAISE prtEnRetard;
			END IF;
			nombrePrtsEnCours := nombrePrtsEnCours + 1;
		END LOOP;

		CLOSE lignesPrts;
		IF nombrePrtsEnCours >= nombreMaximalPrts THEN
			RAISE nombreMaximalAtteint;
		END IF;

	END IF;
	CLOSE leMembre;
EXCEPTION
	WHEN exemplaireNonDisponible THEN
		raise_application_error(-20103,'l''exemplaire n''a pas le statut disponible');
	WHEN nombreMaximalAtteint THEN
		raise_application_error(-20101,'le nombre maximal d''emprunts est atteint');
	WHEN prtEnRetard THEN
		raise_application_error(-20104,'emprunt interdit  cause de retard');
	WHEN	OTHERS THEN
		raise_application_error(-20105,'erreur interne');
END BIPrtEnCoursVrifier;
/
CREATE OR REPLACE TRIGGER BUPrtEnCours
BEFORE UPDATE ON PrtEnCours
BEGIN
	raise_application_error(-20203,'UPDATE interdit pour PrtEnCours');
END BUPrtEnCours;
/
CREATE OR REPLACE TRIGGER AIPrtEnCoursModifStatutEx
AFTER INSERT ON PrtEnCours
REFERENCING
	NEW AS ligneAprs
FOR EACH ROW
BEGIN
	UPDATE 	Exemplaire
	SET 		statut = 'prt'
	WHERE 	idExemplaire = :ligneAprs.idExemplaire;
END AIPrtEnCoursModifStatutEx;
/
-- On utilise le mcanisme de SEQUENCE Oracle pour gnrer les valeurs de
-- la cl primaire noSequence de la table PrtArchiv
CREATE SEQUENCE SequenceNoPrtArchiv
/
CREATE TABLE PrtArchiv
(noSequence				NUMBER(10)		NOT NULL,
 datePrt 				DATE			NOT NULL,
 dateRetour 			DATE			NOT NULL,
 idUtilisateur 			VARCHAR(10)		NOT NULL,
 idExemplaire 			VARCHAR(10)		NOT NULL,
 PRIMARY KEY (noSequence),
 FOREIGN KEY (idUtilisateur) REFERENCES Utilisateur,
 FOREIGN KEY (idExemplaire) REFERENCES Exemplaire,
 CHECK (dateRetour >= datePrt)
)
/
CREATE OR REPLACE TRIGGER ADPrtEnCours
AFTER DELETE ON PrtEnCours
FOR EACH ROW
BEGIN
 	UPDATE 	Exemplaire
 	SET 		statut = 'disponible'
 	WHERE 	idExemplaire = :OLD.idExemplaire;

 	INSERT INTO PrtArchiv VALUES
 		(SequenceNoPrtArchiv.NEXTVAL, :OLD.datePrt, SYSDATE,
		:OLD.idUtilisateur,:OLD.idExemplaire);
	-- SequenceNoPrtArchiv.NEXTVAL retourne la prochaine valeur de la SEQUENCE
END ADPrtEnCours;
/

CREATE TABLE Auteur
(noSequence				NUMBER(10)		NOT NULL,
 nom	 				VARCHAR(20)		NOT NULL,
 prnom 				VARCHAR(20)		NOT NULL,
 PRIMARY KEY (noSequence)
)
/
CREATE TABLE AuteurLivre
(noSequence				NUMBER(10)		NOT NULL,
 ISBN 				CHAR(13)		NOT NULL,
 ordreAuteur 			NUMBER(10)		NOT NULL,
 PRIMARY KEY (noSequence, ISBN),
 UNIQUE (ISBN, ordreAuteur),
 FOREIGN KEY (noSequence) REFERENCES Auteur,
 FOREIGN KEY (ISBN) REFERENCES Livre
)
/
PROMPT Quelques exemples de manipulations de donnes
ALTER SESSION SET NLS_DATE_FORMAT = 'DD-MON-YYYY'
/
ALTER SESSION SET NLS_DATE_LANGUAGE = ENGLISH
/

INSERT INTO Catgorie VALUES('10','Sciences',NULL)
/
INSERT INTO Catgorie VALUES('20','Philosophie',NULL)
/
INSERT INTO Catgorie VALUES('30','Histoire', NULL)
/
INSERT INTO Catgorie VALUES('40','Arts', NULL)
/
INSERT INTO Catgorie VALUES('50','Littrature', NULL) 
/
INSERT INTO Catgorie VALUES('101','Informatique','10')
/
INSERT INTO Catgorie VALUES('102','Mathmatiques','10')
/
INSERT INTO Catgorie VALUES('103','Chimie','10')
/
INSERT INTO Catgorie VALUES('501','Roman','50') 
/
INSERT INTO Catgorie VALUES('502','Posie','50')
/
COMMIT
/
SELECT * FROM Catgorie
/

INSERT INTO Auteur VALUES(1,'Knuth','Donald') 
/
INSERT INTO Auteur VALUES(2,'Salton','Gerard') 
/
INSERT INTO Auteur VALUES(3,'Blaha','Michael') 
/
INSERT INTO Auteur VALUES(4,'Premerlani','Henry F.') 
/
INSERT INTO Auteur VALUES(5,'Hofstadter','Douglas,R.')
/
COMMIT
/
SELECT * FROM Auteur
/

INSERT INTO Livre VALUES('3-333-3333333', 'The Art of Computer Programming',1973, 'Addison Wesley', '101') 
/
INSERT INTO Livre VALUES('0-201-12227-8','Automatic Text Processing',1989, 'Addison Wesley', '101') 
/
INSERT INTO Livre VALUES('0-13-123829-9','Object-Oriented Modeling and Design for DB App.',1998, 'Prentice Hall', '101') 
/
INSERT INTO Livre VALUES('0-394-74502-7', 'Gdel, Escher, Bach : An Eternal Golden Braid', 1980, 'Random House', '20')
/
SELECT * FROM Livre
/

INSERT INTO Editeur VALUES ('Addison Wesley', 'Reading,MA')
/
INSERT INTO Editeur VALUES ('Prentice Hall', 'U.Sad.Riv.')
/
INSERT INTO Editeur VALUES ('Random House', 'NewYork,NY')
/
COMMIT
/
SELECT * FROM Editeur
/

INSERT INTO AuteurLivre VALUES(1, '3-333-3333333', 1)
/
INSERT INTO AuteurLivre VALUES(2, '0-201-12227-8', 1)
/
INSERT INTO AuteurLivre VALUES(3, '0-13-123829-9', 1)
/
INSERT INTO AuteurLivre VALUES(4, '0-13-123829-9', 2)
/
INSERT INTO AuteurLivre VALUES(5, '0-394-74502-7', 1)
/
COMMIT
/

INSERT INTO Exemplaire VALUES(1, '10-dec-1975', 'disponible','3-333-3333333')
/
INSERT INTO Exemplaire VALUES(2, '10-dec-1975', 'disponible','3-333-3333333')
/
INSERT INTO Exemplaire VALUES(3, '15-jan-1990', 'disponible','0-201-12227-8')
/
INSERT INTO Exemplaire VALUES(4, '20-mar-1999', 'disponible', '0-13-123829-9')
/
INSERT INTO Exemplaire VALUES(5, '22-mar-1999', 'disponible', '0-13-123829-9')
/
INSERT INTO Exemplaire VALUES(6, '12-sep-1982', 'disponible', '0-394-74502-7')
/
COMMIT
/
SELECT * FROM Exemplaire;
/

INSERT INTO MembreGnral VALUES(1, 2, 7)
/
COMMIT
/
SELECT * FROM MembreGnral
/

INSERT INTO Utilisateur VALUES(1,'cocorico','Marshal', 'Amanda', 'membre')
/
INSERT INTO Membre VALUES(1,'(111)111-1111')
/

INSERT INTO Utilisateur VALUES(2,'cocorico','Degas', 'Edgar', 'membre')
/
INSERT INTO Membre VALUES(2,'(222)222-2222')
/

INSERT INTO Utilisateur VALUES(3,'cocorico','Lecommis', 'Coco', 'employ')
/
INSERT INTO Employ VALUES(3,'LECC01','commis')
/
INSERT INTO Utilisateur VALUES(4,'cocorico','Lecommis', 'Toto', 'employ')
/
INSERT INTO Employ VALUES(4,'LECT01','commis')
/
SELECT * FROM Utilisateur
/
COMMIT
/

PROMPT Editeur sans Livre
INSERT INTO Editeur VALUES ('Eyrolles','Paris')
/
PROMPT Prt correct
INSERT INTO PrtEnCours VALUES(1,'20-jul-1999',1)
/
PROMPT cas de retard
INSERT INTO PrtEnCours VALUES(3,'30-jul-1999',1)
/
PROMPT cas non disponible
INSERT INTO PrtEnCours VALUES(1,'30-jul-1999',2)
/
DELETE FROM PrtEnCours WHERE idExemplaire = 1
/
SELECT * FROM PrtEnCours
/
SELECT * FROM PrtArchiv
/
PROMPT defaut pour la date de prt
INSERT INTO PrtEnCours(idExemplaire, idUtilisateur) VALUES(1,1)
/
INSERT INTO PrtEnCours(idExemplaire, idUtilisateur) VALUES(3,1)
/
PROMPT cas de maximum atteint (2 dans cet essai)
INSERT INTO PrtEnCours(idExemplaire, idUtilisateur) VALUES(5,1)
/
SELECT * FROM PrtEnCours
/
PROMPT maximum dpass pour Commis
INSERT INTO PrtEnCours(idExemplaire, idUtilisateur) VALUES(2,3)
/
INSERT INTO PrtEnCours(idExemplaire, idUtilisateur) VALUES(4,3)
/
INSERT INTO PrtEnCours(idExemplaire, idUtilisateur) VALUES(5,3)
/

